/* updated d1topgm.c program. Works with gcc.
 * distributed, but not written, by John Beale 10/29/95
 */

#include <stdio.h>
#include <stdlib.h>

char *prog_name;
FILE *fin, *fout;

float skew;

void do_conversion(void);

int main(int argc, char **argv)
{
  prog_name = argv[0];
  if (argc != 4)
    {
      fprintf(stderr, "Usage:\n");
      fprintf(stderr, "\t%s infile.d1 outfile.pgm <num>\n\n", prog_name);
      fprintf(stderr, "where <num> is replaced by a skew calculation factor\n");
      fprintf(stderr, "between 0.00 and 1.00\n");
      exit(1);
    }
  fin = fopen(argv[1], "r");
  if (!fin)
    {
      fprintf(stderr, "%s: cannot open input file: %s\n", prog_name, argv[1]);
      exit(1);
    }
  fout = fopen(argv[2], "wt");
  if (!fout)
    {
      fprintf(stderr, "%s: cannot create output file: %s\n", prog_name, argv[2]);
      exit(1);
    }
  skew = sscanf(argv[3], "%f", &skew);
  do_conversion();
  fclose(fin);
  fclose(fout);
  exit(0);
}

void do_conversion(void)
{
  float x, y;
  int totalprofiles, profiles, *profile, **dem;
  int length = 0, y1, i, j, min, max;
  double minimum, maximum;
  int dum;
  int elevation, cx, cy;

  for(i = 0; i < 4; i++)
    {
      printf("Doing Loop #%d\n", i);
      fscanf(fin, "%f", &x);
      fscanf(fin, "%f", &y);
    }
  fscanf(fin, "%lf", &minimum);
 fscanf(fin, "%lf", &maximum);
  fscanf(fin, "%d", &totalprofiles);

  min = minimum;  /* integer equivalents */
  max = maximum;

  /* ------- Begin of memory allocation -------- */

  profile = (int *) malloc(totalprofiles * sizeof(int));
  if (!profile)
    {
      fprintf(stderr, "%s: memory allocation error\n", prog_name);
      exit(1);
    }
  dem = (int **) malloc(totalprofiles * sizeof(int));
  if(!dem)
    {
      fprintf(stderr, "%s: memory allocation error\n", prog_name);
      exit(1);
    }
  for(i = 0; i < totalprofiles; i++)
    {
      dem[i] = (int *) malloc(totalprofiles * sizeof(int));
      if (!dem[i])
        {
          fprintf(stderr, "%s: memory allocation error\n", prog_name);
          exit(1);
        }
      for (j = 0; j < totalprofiles; j++)
        {
          dem[i][j] = -32767;
        }
    }

  /* --------- End of memory allocation ----------- */

  fprintf(stdout, "Minimum = %f\n", minimum);
  fprintf(stdout, "Maximum = %f\n", maximum);
  fprintf(stdout, "Number of Profiles = %d\n", totalprofiles);

  fprintf(stdout, "Loading in the File.....\n");

  for(i = 0; i < totalprofiles; i++)
    {
      fscanf(fin, "%d", &dum); /* read in profile number */
      fscanf(fin, "%d", &profiles);  /* read in data points in profile */
      if (length < profiles)
        {
          length = profiles;
        }
      fscanf(fin, "%f %f", &x, &y); /* read in x and y coordinates */
      for(j = 0; j < profiles; j++)
        {
          fscanf(fin, "%d", &elevation);
          dem[i][j] = elevation;
        }
    }

  fprintf(stdout, "Trying to Rearrange the Skew...\n");
  /* rearrange the western edge such that the skew is correct */
  for(cx = (int) (skew * totalprofiles); cx < totalprofiles; cx++)
  /* Go across the row to check */
    {
      fprintf(stdout, "Checking Profile #%d\r", cx);
      for(cy = 0; cy < length; cy++)
        {
          if(dem[cx][cy] == -32767)
            {
              fprintf(stdout, "Rearranging Profile %d\n", cx);
              for(y1 = 0; y1 < length; y1++)    /* temporarily make a new profile
 */
                {
                  profile[y1] = dem[cx][y1];
                }
              for(y1 = length; y1 > -1; y1--)
                {
                  dem[cx][y1] = profile[length-y1];
                }
              break;
            }
        }
    }
  /* even out the data to the left of center */
  fprintf(stdout, "Evening out the data....\n");

  for(cy = 0; cy < length; cy++)  /* go down from N to S */
    {
      for(cx = 150; cx > -1; cx--)   /* Go west from the 150 profile */
        {
          if(dem[cx][cy] == -32767)
            {
              if(dem[cx+1][cy] == -32767)
                {
                  dem[cx+1][cy] = minimum;
                }
              dem[cx][cy] = dem[cx+1][cy];
            }
        }
      for(cx = 150; cx < totalprofiles; cx++) /* Go East from the 150 profile */
        {
          if(dem[cx][cy] == -32767)
            {
              if(dem[cx-1][cy] == -32767)
                {
                  dem[cx-1][cy] = minimum;
                }
              dem[cx][cy] = dem[cx-1][cy];
            }
        }
    }

  fprintf(stdout, "Writing the data to the output: [%d x %d] PGM\n", totalprofiles, length);

  fprintf(fout, "P2\n");    /* PGM ASCII header */
  fprintf(fout, "%d %d\n", totalprofiles, length); /* xsize, ysize of file */
  fprintf(fout, "%d\n", max-min);  /* maximum value of any pixel */
  for(cy = 0; cy < length; cy++) /* routine to write out the PGM file */
    {
      for(cx = 0; cx < (totalprofiles - 1); cx++)
        {
          fprintf(fout, "%d ", dem[cx][cy] - min);
        }
      fprintf(fout, "%d\n", dem[totalprofiles - 1][cy] - min);
    }
  /* --------- free the allocated memory --------- */
  free(profile);
  for (i = 0; i < totalprofiles; i++)
    {
      free(dem[i]);
    }
  free(dem);
}







